using System;
using System.Linq;
using System.Threading.Tasks;
using Avalonia;
using Avalonia.Controls;
using Avalonia.Controls.ApplicationLifetimes;
using Avalonia.Markup.Xaml;
using Avalonia.Media;
using Avalonia.Platform.Storage;
using Tesses.CMS.Avalonia.ViewModels;
using Tesses.CMS.Avalonia.Views;
using Tesses.CMS.Client;
using Avalonia.Media.Imaging;
using Avalonia.Platform;
using Tesses.VirtualFilesystem;
using Tesses.VirtualFilesystem.Extensions;
using System.IO;
using System.Collections.Generic;
using System.Threading;
using Tesses.CMS.Avalonia.Models;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
namespace Tesses.CMS.Avalonia;

public partial class App : Application
{
    public static OpenedCtrls OpenedControls {get;}=new OpenedCtrls();
    public static IPlatform Platform {get;set;} = new NullPlatform();

    public static MainWindow? Window {get;set;}

    public static TessesCMSClient Client {get;} = new TessesCMSClient();

    public static MainView? MainView {get;set;}
    
    public override void Initialize()
    {
        
        AvaloniaXamlLoader.Load(this);
    }

    public static List<ActiveDownload> Downloads {get;}=new List<ActiveDownload>();

    public static void AddActiveDownload(ActiveDownload dl)
    {
        lock(Downloads)
        {
            if(!Downloads.Any(e=>e.Path ==  dl.Path)) 
            {
                Downloads.Add(dl);
                dl.Start().Wait(0);
            }
        }
    }

    public override void OnFrameworkInitializationCompleted()
    {
        Client.RootUrl = Platform.Configuration.ServerUrl;
        string title = GetApplicationTitle();
        if (ApplicationLifetime is IClassicDesktopStyleApplicationLifetime desktop)
        {
            Window = new MainWindow
            {
                DataContext = new MainViewModel(title),
                Title = title
            };
            desktop.MainWindow = Window;
        }
        else if (ApplicationLifetime is ISingleViewApplicationLifetime singleViewPlatform)
        {
            MainView = new MainView
            {
                DataContext = new MainViewModel(title)
            };
            singleViewPlatform.MainView = MainView;
        }

        base.OnFrameworkInitializationCompleted();
    }

    internal static string GetApplicationTitle()
    {
        string title="Tesses CMS";
        Task.Run(async()=>{
            try {
                var data = await Client.GetBrandingAsync();
                title = data.Title;
            }catch(Exception ex)
            {
                _=ex;
            }
        }).Wait();
        return title;
    }

    internal static async Task<IImage> GetMovieThumbnailAsync(string username, string name)
    {
        //we need to cache the resource
        if(Platform.Configuration.CacheResources && Platform.DownloadFilesystem != null)
        {
            UnixPath cacheDir = Special.Root / "Metadata" / "Cache" / username / "Movies" / name;
            Log($"About to create directory: {cacheDir}");
         
            await Platform.DownloadFilesystem.CreateDirectoryAsync(cacheDir);
            Log($"Created directory: {cacheDir}");
            UnixPath thumbnailPath = cacheDir / "thumbnail.jpg";
            
            if(await Platform.DownloadFilesystem.FileExistsAsync(thumbnailPath))
            {
                Log($"Image exists: {thumbnailPath}");
                using(var sr = await Platform.DownloadFilesystem.OpenReadAsync(thumbnailPath))
                {
                    MemoryStream ms = new MemoryStream();
                    sr.CopyTo(ms);
                    ms.Position=0;
                    Log($"Image read from file: {thumbnailPath}");
                    return new Bitmap(ms);
                }
                
            }
            else
            {
                var metadata = await Client.Movies.GetMovieContentMetadataAsync(username,name);
                if(metadata.HasThumbnail)
                {
                    using(var strm =  await Platform.DownloadFilesystem.OpenAsync(thumbnailPath,System.IO.FileMode.Create,System.IO.FileAccess.Write,System.IO.FileShare.None))
                    {
                        MemoryStream ms=new MemoryStream();
                        Log($"About to read from network and save: {thumbnailPath}");

                        await Client.Movies.DownloadThumbnailAsync(username,name,ms);
                        ms.Position=0;                        
                        ms.CopyTo(strm);
                        ms.Position=0;
                        Log($"Image read from network and saved: {thumbnailPath}");
                        return new Bitmap(ms);
                    }
                }
                
            }
        }
        else
        {
             var metadata = await Client.Movies.GetMovieContentMetadataAsync(username,name);
                if(metadata.HasThumbnail)
                {
                    MemoryStream ms=new MemoryStream();  
                        Log($"About to read from network: {username} {name}");
   
                    await Client.Movies.DownloadThumbnailAsync(username,name,ms);
                    ms.Position = 0;
                                            Log($"Image read from network: {username} {name}");

                    return new Bitmap(ms);
                    
                }
                
        }
        Log($"Image does not exist: {username} {name}");

        return new Bitmap(AssetLoader.Open(new Uri("avares://Tesses.CMS.Avalonia/Assets/no-media-icon.png")));
    }
    
    internal static async Task<IImage> GetShowThumbnailAsync(string username, string name)
    {
        //we need to cache the resource
        if(Platform.Configuration.CacheResources && Platform.DownloadFilesystem != null)
        {
            UnixPath cacheDir = Special.Root / "Metadata" / "Cache" / username / "Shows" / name;
            Log($"About to create directory: {cacheDir}");
         
            await Platform.DownloadFilesystem.CreateDirectoryAsync(cacheDir);
            Log($"Created directory: {cacheDir}");
            UnixPath thumbnailPath = cacheDir / "thumbnail.jpg";
            
            if(await Platform.DownloadFilesystem.FileExistsAsync(thumbnailPath))
            {
                Log($"Image exists: {thumbnailPath}");
                using(var sr = await Platform.DownloadFilesystem.OpenReadAsync(thumbnailPath))
                {
                    MemoryStream ms = new MemoryStream();
                    sr.CopyTo(ms);
                    ms.Position=0;
                    Log($"Image read from file: {thumbnailPath}");
                    return new Bitmap(ms);
                }
                
            }
            else
            {
                var metadata = await Client.Shows.GetShowContentMetadataAsync(username,name);
                if(metadata.HasThumbnail)
                {
                    using(var strm =  await Platform.DownloadFilesystem.OpenAsync(thumbnailPath,System.IO.FileMode.Create,System.IO.FileAccess.Write,System.IO.FileShare.None))
                    {
                        MemoryStream ms=new MemoryStream();
                        Log($"About to read from network and save: {thumbnailPath}");

                        await Client.Shows.DownloadThumbnailAsync(username,name,ms);
                        ms.Position=0;                        
                        ms.CopyTo(strm);
                        ms.Position=0;
                        Log($"Image read from network and saved: {thumbnailPath}");
                        return new Bitmap(ms);
                    }
                }
                
            }
        }
        else
        {
             var metadata = await Client.Shows.GetShowContentMetadataAsync(username,name);
                if(metadata.HasThumbnail)
                {
                    MemoryStream ms=new MemoryStream();  
                        Log($"About to read from network: {username} {name}");
   
                    await Client.Shows.DownloadThumbnailAsync(username,name,ms);
                    ms.Position = 0;
                                            Log($"Image read from network: {username} {name}");

                    return new Bitmap(ms);
                    
                }
                
        }
        Log($"Image does not exist: {username} {name}");

        return new Bitmap(AssetLoader.Open(new Uri("avares://Tesses.CMS.Avalonia/Assets/no-media-icon.png")));
    }


    public static void Log(string text)
    {
        lock(App.Platform)
        {
            var fs=App.Platform.DownloadFilesystem;
            if(fs != null && App.Platform.Configuration.Log)
            {
                
                using(var strm = fs.Open(Special.Root/"Logs.txt",FileMode.Append,FileAccess.Write,FileShare.None))
                {
                    using(var sw = new StreamWriter(strm))
                    {
                        sw.WriteLine(text);
                    }
                }
            }
        }
    }

    internal static void StartDownload(UserAccount account, Movie movie)
    {
        string username = account.Username;

        FavoritesItem item =new FavoritesItem()
        {
            Username = account.Username,
            UserProperName = account.ProperName,
            Account = account,
            Media = JObject.FromObject(movie),
            MediaName = movie.Name,
            MediaProperName = movie.ProperName,
            Type = FavoriteType.Movie
        };

        App.AddDownload(item);
        

        UnixPath dir = Special.Root / "Downloads" / username / "Movies" / movie.Name;
        UnixPath incomplete = dir / movie.ProperName + ".mp4.part";
        UnixPath filename = dir / movie.ProperName + ".mp4";
        Platform.DownloadFilesystem?.CreateDirectory(dir);

        if(Platform.DownloadFilesystem != null)
        {
            ActiveDownload download = new ActiveDownload();
            download.Name = movie.ProperName;
            download.Path = filename.Path;
            download.Start = async()=>{
                
                if(!await Platform.DownloadFilesystem.FileExistsAsync(filename))
                {

                    DateTime last = DateTime.Now;
                   
                    using(var strm = await Platform.DownloadFilesystem.OpenAsync(incomplete,FileMode.OpenOrCreate,FileAccess.Write,FileShare.None,download.TokenSource.Token))
                    {
                        await Client.Movies.DownloadMovieAsync(username,movie.Name,strm,download.TokenSource.Token,new Progress<double>(e=>{
                            
                            var now = DateTime.Now;

                            if(last.AddSeconds(0.5) < now)
                            {
                            
                             download.UpdateProgress(e);
                             last = now;
                            }


                        }));
                        
                    }
                    if(!download.TokenSource.IsCancellationRequested)
                    await Platform.DownloadFilesystem.MoveFileAsync(incomplete,filename);
                    RemoveActiveDownload(download);
                }
            };
            AddActiveDownload(download);
        }
        
    }

    internal static void AddDownload(FavoritesItem item)
    {
        if(App.Platform.DownloadFilesystem == null) return;
        List<FavoritesItem> items = new List<FavoritesItem>();

        foreach(var _item in EnumerateDownloads())
            if(_item.Equals(item)) return;
            else items.Add(_item);
        
        items.Add(item);

        App.Platform.DownloadFilesystem.CreateDirectory(Special.Root / "Metadata");
        App.Platform.DownloadFilesystem.WriteAllText(Special.Root / "Metadata" / "Downloads.json",JsonConvert.SerializeObject(items));
  
    }

    private static void RemoveActiveDownload(ActiveDownload download)
    {
        lock(Downloads)
        {
            Downloads.Remove(download);
            download.SendDone();
            
        }
    }

    internal static List<ActiveDownload> GetActiveDownloads()
    {
        lock(Downloads) {
            return Downloads.ToList();
        }
    }

    internal static IEnumerable<FavoritesItem> EnumerateFavorites()
    {
        if(App.Platform.DownloadFilesystem == null) yield break;

        if(!App.Platform.DownloadFilesystem.FileExists(Special.Root / "Metadata" / "Favorites.json")) yield break;

        App.Platform.DownloadFilesystem.CreateDirectory(Special.Root / "Metadata");

        string text = App.Platform.DownloadFilesystem.ReadAllText(Special.Root / "Metadata" / "Favorites.json");

        var res=JsonConvert.DeserializeObject<List<FavoritesItem>>(text);
        if(res != null) 
            foreach(var item in res)
                yield return item;
    }
     internal static IEnumerable<FavoritesItem> EnumerateDownloads()
    {
        if(App.Platform.DownloadFilesystem == null) yield break;

        if(!App.Platform.DownloadFilesystem.FileExists(Special.Root / "Metadata" / "Downloads.json")) yield break;

        App.Platform.DownloadFilesystem.CreateDirectory(Special.Root / "Metadata");

        string text = App.Platform.DownloadFilesystem.ReadAllText(Special.Root / "Metadata" / "Downloads.json");

        var res=JsonConvert.DeserializeObject<List<FavoritesItem>>(text);
        if(res != null) 
            foreach(var item in res)
                yield return item;
    }
    internal static bool FavoritesExists(FavoritesItem item)
    {
        foreach(var _item in EnumerateFavorites())
            if(_item.Equals(item)) return true;
        return false;
    }
    internal static void AddFavorite(FavoritesItem item)
    {
        if(App.Platform.DownloadFilesystem == null) return;
        List<FavoritesItem> items = new List<FavoritesItem>();

        foreach(var _item in EnumerateFavorites())
            if(_item.Equals(item)) return;
            else items.Add(_item);
        
        items.Add(item);

        App.Platform.DownloadFilesystem.CreateDirectory(Special.Root / "Metadata");
        App.Platform.DownloadFilesystem.WriteAllText(Special.Root / "Metadata" / "Favorites.json",JsonConvert.SerializeObject(items));
    }

    internal static void RemoveFavorite(FavoritesItem item)
    {
        if(App.Platform.DownloadFilesystem == null) return;
        List<FavoritesItem> items = new List<FavoritesItem>();

        foreach(var _item in EnumerateFavorites())
            if(_item.Equals(item)) continue;
            else items.Add(_item);
            
        App.Platform.DownloadFilesystem.CreateDirectory(Special.Root / "Metadata");
        App.Platform.DownloadFilesystem.WriteAllText(Special.Root / "Metadata" / "Favorites.json",JsonConvert.SerializeObject(items));
  
    }

    internal static void LaunchUrl(string v)
    {
        Platform.LaunchUrl(v);
    }
}

public class ActiveDownload
{
    public CancellationTokenSource TokenSource {get;set;}=new CancellationTokenSource();

    public string Path {get;set;}="";
    

    public string Name {get;set;}="";

    public double Progress {get;set;}=0;

    public event Action<double>? ProgressChanged=null;

    public event Action? Done = null;

    public void UpdateProgress(double d)
    {
       
        Progress = d;
        ProgressChanged?.Invoke(d);
    }

    public void SendDone()
    {
        Done?.Invoke();
    }


    public Func<Task> Start {get;set;}=async()=>{await Task.CompletedTask;};
}

public class OpenedCtrls
{
    List<IDisposable> disposables=new List<IDisposable>();

    public void Register(IDisposable disposable)
    {
        disposables.Add(disposable);
    }

    public void Destroy()
    {
        foreach(var item in disposables) item.Dispose();

        disposables.Clear();
    }
}