using System.Threading.Tasks;
using Avalonia.Controls;
using LibVLCSharp.Shared;
using Tesses.VirtualFilesystem;

public interface IPlatform
{
    Configuration Configuration {get;}
    Task BrowseForDownloadDirectoryAsync();

    bool PlatformUsesNormalPathsForDownload {get;}

    IVirtualFilesystem? DownloadFilesystem {get;}
    bool CanTakeScreenShots { get; }
    string ScreenshotPath { get; }
    bool MustMoveScreenshot { get; }

    Task WriteConfigurationAsync();

    Task ReadConfigurationAsync();

    Control CreatePlayer();

    void SetMediaPlayer(Control control, MediaPlayer? mediaPlayer);

    MediaPlayer? GetMediaPlayer(Control control);

    void LaunchUrl(string url);
 
}

public class NullPlatform : IPlatform
{
    public bool PlatformUsesNormalPathsForDownload => false;

    public IVirtualFilesystem? DownloadFilesystem 
    {
        get => null;
        set => _=value;
    }

    Configuration _conf=new Configuration();

    public Configuration Configuration => _conf;

    public bool CanTakeScreenShots => false;

    public string ScreenshotPath => throw new System.NotImplementedException();

    public bool MustMoveScreenshot => throw new System.NotImplementedException();

    public async Task BrowseForDownloadDirectoryAsync()
    {
        await Task.CompletedTask;
    }

    public async Task ReadConfigurationAsync()
    {
        await Task.CompletedTask;
    }

    public async Task WriteConfigurationAsync()
    {
        await Task.CompletedTask;
    }

    public Control CreatePlayer()
    {
        return new TextBlock(){Text="Video player unavailable"};
    }

    public void SetMediaPlayer(Control control, MediaPlayer? mediaPlayer)
    {
        
    }

    public MediaPlayer? GetMediaPlayer(Control control)
    {
        return null;
    }

    public void LaunchUrl(string url)
    {
        
    }
}