namespace Tesses.CMS.Avalonia.ViewModels;

using System;
using System.Threading.Tasks;
using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using CommunityToolkit.Mvvm.Messaging;
using Tesses.CMS.Avalonia.Models;
using Tesses.CMS.Avalonia.ViewModels.HomePages;
using Tesses.CMS.Client;
using Tesses.VirtualFilesystem;
using Tesses.VirtualFilesystem.Extensions;

public partial class HomePageViewModel : ViewModelBase
{
    public HomePageViewModel()
    {
        _curPage = new HomeUserListPageViewModel(this);
    }
    private ViewModelBase _curPage;
    public ViewModelBase CurrentPage
    {
        get
        {
            return _curPage;
        }
        set{
            App.OpenedControls.Destroy();
            _curPage = value;
            var disp = value as IDisposable;
            if(disp != null) App.OpenedControls.Register(disp);
            OnPropertyChanged(nameof(CurrentPage));

        }
    }

    [RelayCommand]
    private void Back()
    {
        var page = CurrentPage as IBackable;
        if(page != null)
        {
            CurrentPage = page.Back();
        }
        else
        {
            //can't go back
        }
    }

    public async Task NavigateToAsync(FavoritesItem item)
    {
        switch(item.Type)
        {
            case FavoriteType.Movie:
                var homeUserList = new HomeUserListPageViewModel(this);
                
                
                
                var homeUser = new HomeUserPageViewModel(this,homeUserList,item.Account);
                
                var homeMovieList = new HomeMovieListPageViewModel(this,homeUser);
                Movie movie = item.Media.ToObject<Movie>() ?? new Movie(){Name=item.MediaName,ProperName = item.MediaProperName};
                MovieItem movieItem=new MovieItem(movie,await App.GetMovieThumbnailAsync(item.Username,item.MediaName));

                var homeMovie = new HomeMoviePageViewModel(this,homeMovieList,item.Account,movieItem);
                CurrentPage = homeMovie;
                break;
        }
    }

    public async Task PlayDownloadedAsync(FavoritesItem item)
    {
        if(App.Platform.DownloadFilesystem == null) return;
        switch(item.Type)
        {
            case FavoriteType.Movie:
                UnixPath dir = Special.Root / "Downloads" / item.Username / "Movies" / item.MediaName;
                UnixPath filename = dir / item.MediaProperName + ".mp4";
                if(!await App.Platform.DownloadFilesystem.FileExistsAsync(filename)) return;
                
                var homeUserList = new HomeUserListPageViewModel(this);
                
                
                
                var homeUser = new HomeUserPageViewModel(this,homeUserList,item.Account);
                
                var homeMovieList = new HomeMovieListPageViewModel(this,homeUser);
                Movie movie = item.Media.ToObject<Movie>() ?? new Movie(){Name=item.MediaName,ProperName = item.MediaProperName};
                MovieItem movieItem=new MovieItem(movie,await App.GetMovieThumbnailAsync(item.Username,item.MediaName));

                var homeMovie = new HomeMoviePageViewModel(this,homeMovieList,item.Account,movieItem);
                 CurrentPage = new HomeMovieVideoPlayerViewModel(this,homeMovie,await App.Platform.DownloadFilesystem.OpenReadAsync(filename));
                break;
        }
    }
}