

using System;
using System.Collections.ObjectModel;
using System.Threading.Tasks;
using Avalonia.Media;
using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using CommunityToolkit.Mvvm.Messaging;
using Newtonsoft.Json.Linq;
using ReactiveUI;
using Tesses.CMS.Avalonia.Models;
using Tesses.CMS.Avalonia.Views.HomePages;
using Tesses.CMS.Client;
namespace Tesses.CMS.Avalonia.ViewModels.HomePages;

public partial class HomeMoviePageViewModel : ViewModelBase, IBackable
{
    HomePageViewModel home;
    
    HomeMovieListPageViewModel movieList;

    Movie movie;

    [ObservableProperty]
    private IImage? _thumbnail=null;

    [ObservableProperty]
    private string _movieTitle;

    [ObservableProperty]
    private string _movieCreator;
    [ObservableProperty]
    private string _description;
    [ObservableProperty]
    private string _published;

    [ObservableProperty]
    private string _modified;

    string? watchUrl=null;


    [ObservableProperty]
    private bool _canWatch=false;

    [ObservableProperty]
    private bool _canDownload=false;

    [ObservableProperty]

    private string _addOrRemoveFromFavorites="";
    private bool _favoriteExists=false;
    public bool FavoriteExists {
        get=>_favoriteExists;
        set{
            _favoriteExists = value;
            AddOrRemoveFromFavorites = value ? "Remove from Favorites" : "Add to Favorites";
        }
    }

    string username;
    UserAccount account;

    FavoritesItem fav;

    public HomeMoviePageViewModel(HomePageViewModel home,HomeMovieListPageViewModel movieList,UserAccount account,MovieItem movie)
    {
        this.account = account;
        username = account.Username;
        this.home = home;
        this.movie = movie.Movie;
        this.movieList = movieList;

        this.Thumbnail = movie.Image; 
        this.MovieTitle = movie.Movie.ProperName;
        this.MovieCreator = account.ProperName;
        this.Description = movie.Movie.Description;
        this.Modified = $"Last Modified: {movie.Movie.LastUpdated.ToShortDateString()}";
        this.Published = $"Published: {movie.Movie.CreationTime.ToShortDateString()}";
        fav=new FavoritesItem()
        {
            Type = FavoriteType.Movie,
            Username = account.Username,
            UserProperName = account.ProperName,
            MediaName = movie.Movie.Name,
            MediaProperName = movie.Movie.ProperName,
            Account = account,
            Media = JObject.FromObject(movie.Movie)
        };
        FavoriteExists = App.FavoritesExists(fav);
        
        Task.Run(async()=>{
            var res=await App.Client.Movies.GetMovieContentMetadataAsync(username,movie.Movie.Name);
            if(res.HasBrowserStream)
                watchUrl = res.BrowserStream;
            CanWatch = res.HasBrowserStream;
            
            CanDownload = res.HasDownloadStream;
            
            
        }).Wait(0);
    }

    
    public ViewModelBase Back()
    {
        return movieList;
    }
    
    [RelayCommand]
    public void Watch()
    {
        if(!string.IsNullOrWhiteSpace(watchUrl))
        {
            home.CurrentPage = new HomeMovieVideoPlayerViewModel(home,this,watchUrl);
        }
    }
    [RelayCommand]
    public void Download()
    {
        if(CanDownload)
        App.StartDownload(account,movie);
    }

    [RelayCommand]
    public void AddToFavorites()
    {
        if(FavoriteExists)
        {
            App.RemoveFavorite(fav);
        }
        else
        {
            App.AddFavorite(fav);
        }
        FavoriteExists = !FavoriteExists;
    }
}