﻿namespace Tesses.CMS.Avalonia.ViewModels;

using System;
using System.Collections.ObjectModel;
using System.Linq;
using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using CommunityToolkit.Mvvm.Messaging;
using Tesses.CMS.Avalonia.Models;

public partial class MainViewModel : ViewModelBase
{
    public MainViewModel(string title)
    {
        _pages.Add(new Page("Home",()=>new HomePageViewModel()));
        _pages.Add(new Page("Favorites",()=>new FavoritesPageViewModel(this)));
        _pages.Add(new Page("Notifications",()=>new NotificationsPageViewModel()));
        _pages.Add(new Page("Downloads",()=>new DownloadsPageViewModel(this)));
        _pages.Add(new Page("Video Player",()=>new VideoPlayerViewModel()));
        _pages.Add(new Page("Settings",()=>new SettingsPageViewModel(this)));
        SelectedListItem = Pages.First();
        this.Title = title;

        if(Environment.GetCommandLineArgs().Length > 1)
        {
            string path = Environment.GetCommandLineArgs()[1];
            Setting = true;
            SelectedListItem = _pages.First(e=>e.Name == "Video Player");
            CurrentPage = new VideoPlayerViewModel(path);
            PaneOpen=false;
            Setting = false;
        }
    }
    [ObservableProperty]
    private bool _paneOpen=true;
     private ViewModelBase _curPage=new HomePageViewModel();
    public ViewModelBase CurrentPage
    {
        get
        {
            return _curPage;
        }
        set{
            App.OpenedControls.Destroy();
            _curPage = value;
            var disp = value as IDisposable;
            if(disp != null) App.OpenedControls.Register(disp);
            OnPropertyChanged(nameof(CurrentPage));

        }
    }

    [ObservableProperty]
    private ObservableCollection<Page> _pages=new ObservableCollection<Page>();
    [ObservableProperty]
    private Page? _selectedListItem;
    [ObservableProperty]
    private string _loginText="Login";

    [ObservableProperty]
    private string _title;

    public bool Setting {get;set;}

     partial void OnSelectedListItemChanged(Page? value)
    {
        if (value is null) return;
        if(Setting) return;
        
        

        CurrentPage = value.ViewModel;
    }

    [RelayCommand]
    private void TogglePane()
    {
        PaneOpen = !PaneOpen;
    }
    [RelayCommand]
    private void LoginAccount()
    {
        App.Log("Login button");
    }

    internal void SetHome()
    {
        SelectedListItem = Pages[0];   
    }
}
