namespace Tesses.CMS.Avalonia.ViewModels;

using System;
using System.Collections.ObjectModel;
using System.Threading.Tasks;
using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using CommunityToolkit.Mvvm.Messaging;
using global::Avalonia.Threading;
using LibVLCSharp.Shared;
using ReactiveUI;
using Tesses.CMS.Avalonia.Models;
using Tesses.CMS.Client;
using Tesses.VirtualFilesystem;

public partial class SavedDownloadViewModel : ViewModelBase
{
     FavoritesItem item;
     MainViewModel model;
    
    public SavedDownloadViewModel(FavoritesItem item,MainViewModel model)
    {
        this.item=item;
        this.model = model;
        DownloadOrViewCommand = ReactiveCommand.CreateFromTask(DownloadOrViewAsync);
        UpdateDownloadButton();
    }
    public string Name => item.MediaProperName;

    public string AuthorName => item.UserProperName;

    [ObservableProperty]
    private string _downloadBtnText = "Download";

    bool downloaded=false;

    private void UpdateDownloadButton()
    {
        switch(item.Type)
        {
            case FavoriteType.Movie:
                UnixPath dir = Special.Root / "Downloads" / item.Username / "Movies" / item.MediaName;
                UnixPath filename = dir / item.MediaProperName + ".mp4";
                downloaded=(App.Platform.DownloadFilesystem?.FileExists(filename) ?? false);
                DownloadBtnText=downloaded ? "View" : "Download";
                break;
        }
    }
    
    public IReactiveCommand DownloadOrViewCommand {get;}
    
    public async Task DownloadOrViewAsync()
    {
        UpdateDownloadButton();
        if(downloaded)
        {
            model.SetHome();
            HomePageViewModel hpvm=new ();
            model.CurrentPage = hpvm;
            await hpvm.PlayDownloadedAsync(item);
            return;
        }
        switch(item.Type)
        {
            case FavoriteType.Movie:
                App.StartDownload(item.Account,item.Media.ToObject<Movie>() ?? new Movie(){Name=item.MediaName,ProperName=item.MediaProperName});
                break;
        }
        UpdateDownloadButton();
    }
}