using System;
using System.IO;
using Avalonia;
using Avalonia.Controls;
using Avalonia.Data;
using Avalonia.Threading;
using LibVLCSharp.Shared;
using Tesses.CMS.Avalonia;
using Tesses.CMS.Avalonia.Views;
using Tesses.VirtualFilesystem;
using Tesses.VirtualFilesystem.Extensions;

namespace Tesses.CMS.Avalonia.Views;
    public static class MMS
    {
        public static void MoveScreenshot(this IPlatform p,string path)
        {
            if(p.DownloadFilesystem == null) return;
            p.DownloadFilesystem.CreateDirectory(Special.Root/"Screenshots");
            using(var ms = File.OpenRead(path))
            {
                using(var f = p.DownloadFilesystem.OpenWrite(Special.Root/"Screenshots"/Path.GetFileName(path)))
                {
                    ms.CopyTo(f);
                }   
            }
            File.Delete(path);
        }
    }

public class VideoPlayer : Grid
{
      /// <summary>
        /// MediaPlayer Data Bound property
        /// </summary>
        /// <summary>
        /// Defines the <see cref="MediaPlayer"/> property.
        /// </summary>
        public static readonly DirectProperty<VideoPlayer, MediaPlayer?> MediaPlayerProperty =
            AvaloniaProperty.RegisterDirect<VideoPlayer, MediaPlayer?>(
                nameof(MediaPlayer),
                o => o.MediaPlayer,
                (o, v) => o.MediaPlayer = v,
                defaultBindingMode: BindingMode.TwoWay);

    public VideoPlayer()
    {
        view=App.Platform.CreatePlayer();

        
        
        this.RowDefinitions.Add(new RowDefinition(GridLength.Star));
        this.RowDefinitions.Add(new RowDefinition(GridLength.Auto));
        this.Children.Add(view);
        SetRow(view,0);
        Grid seekPanel = new Grid();
        Button ss=new Button();
        ss.Content = "SS";
        ss.Click += (sender,e)=>{
            //take screenshot
            App.Log($"CanTakeScreenShots: {App.Platform.CanTakeScreenShots}");
            if(!App.Platform.CanTakeScreenShots) return;

            string realDir =App.Platform.ScreenshotPath; //App.Platform.DownloadFilesystem == null ? Environment.GetFolderPath(Environment.SpecialFolder.MyPictures) : Path.Combine(App.Platform.Configuration.DownloadPath,"Screenshots");
            App.Log($"ScreenshotPath: {App.Platform.ScreenshotPath}");
            Directory.CreateDirectory(realDir);
            if(MediaPlayer == null) return;
            string name = Path.Combine(realDir,$"TessesCMS-Screenshot-{DateTime.Now.ToString("yyyyMMdd_HHmmss")}-{TimeSpan.FromMilliseconds(MediaPlayer.Position*MediaPlayer.Length).ToString().Replace(":","_")}.png");
            App.Log($"name: {name}");
            MediaPlayer.TakeSnapshot(0,name,0,0);
            if(App.Platform.MustMoveScreenshot)
                App.Platform.MoveScreenshot(name);
            App.Log("Screenshot taken");
        };
        slider = new Slider();
        slider.Minimum = 0;
        slider.Maximum = 1000000;
        
        slider.ValueChanged+= (sender,e)=>{
            if(MediaPlayer != null)
            {
                if(!theyareseeking)
                MediaPlayer.Position = (float)(slider.Value/slider.Maximum);
            }
        };
        playBtn=new Button();
        playBtn.Content = "Play";
        playBtn.Click += (sender,e)=>{
            if(MediaPlayer != null)
            {
                if(MediaPlayer.IsPlaying)
                {
                    
                    MediaPlayer.Pause();
                }
                else
                {
                    MediaPlayer.Play();
                }
            }
        };
       
       
        seekPanel.ColumnDefinitions.Add(new ColumnDefinition(GridLength.Auto));
        seekPanel.ColumnDefinitions.Add(new ColumnDefinition(GridLength.Star));
        seekPanel.ColumnDefinitions.Add(new ColumnDefinition(GridLength.Auto));
        seekPanel.Children.Add(playBtn);
        seekPanel.Children.Add(slider);
        seekPanel.Children.Add(ss);
        SetColumn(playBtn,0);
        SetColumn(slider,1);
        SetColumn(ss,2);
        this.Children.Add(seekPanel);
        SetRow(seekPanel,1);
    }
    Slider slider;
    Button playBtn;
    Control view;
    public MediaPlayer? MediaPlayer 
    {
        get=>App.Platform.GetMediaPlayer(view);
        set{
            var mp = App.Platform.GetMediaPlayer(view);
            if(mp != null)
            {
                mp.Paused -= Paused;
                mp.Playing -= Playing;
                mp.PositionChanged -= PositionChanged;
            }
            App.Platform.SetMediaPlayer(view,value);
            if(value != null)
            {
                value.Paused += Paused;
                value.Playing += Playing;
                value.PositionChanged += PositionChanged;
            }
        }
    }



    bool theyareseeking=false;
  

    private void PositionChanged(object? sender, MediaPlayerPositionChangedEventArgs e)
    {
Dispatcher.UIThread.Invoke(()=>{
    theyareseeking=true;
        slider.Value = (e.Position*slider.Maximum);
    theyareseeking=false;
});
    }



    private void Playing(object? sender, EventArgs e)
    {
        Dispatcher.UIThread.Invoke(()=>{
        playBtn.Content="Pause";
        });
    }

    private void Paused(object? sender, EventArgs e)
    {
        Dispatcher.UIThread.Invoke(()=>{
        playBtn.Content="Play";
        });
    }
}